﻿using MySql.Data.MySqlClient;
using SqlSugar;
using System;
using System.Collections.Generic;
using System.Linq;
using System.Linq.Expressions;
using System.Threading.Tasks;
using TEAMModelOS.SDK.Context.Exception;
using TEAMModelOS.SDK.Extension.DataResult.PageToken;

namespace CMS.Context
{
    public class TargetDBContext<Entity> where Entity : class, new()
    {
        public SqlSugarClient Db;
        /// <summary>
        /// 修改后的代码
        /// </summary>
        /// <returns></returns>
        public static TargetDBContext<Entity> OpDB()
        {
            TargetDBContext<Entity> dbcontext_t = new TargetDBContext<Entity>();
            dbcontext_t.Db = new SqlSugarClient(new ConnectionConfig()
            {
                ConnectionString = BaseConfigModel.Configuration["DbConnection:TargetMySqlConnection"],
                DbType = SqlSugar.DbType.MySql,
                IsAutoCloseConnection = true,
                InitKeyType = InitKeyType.Attribute
            });
            return dbcontext_t;
        }
        protected TargetDBContext()
        {
            Db = new SqlSugarClient(new ConnectionConfig()
            {
                ConnectionString = BaseConfigModel.Configuration["DbConnection:TargetMySqlConnection"],
                DbType = SqlSugar.DbType.MySql,
                IsAutoCloseConnection = true,
                InitKeyType = InitKeyType.Attribute
            });
            //调式代码 用来打印SQL
            Db.Aop.OnLogExecuting = (sql, pars) =>
            {
                Console.WriteLine(sql + "\r\n" +
                    Db.Utilities.SerializeObject(pars.ToDictionary(it => it.ParameterName, it => it.Value)));
            };
        }
        public void Dispose()
        {
            if (Db != null)
            {
                Db.Dispose();
            }
        }
        public SimpleClient<Entity> CurrentDb { get { return new SimpleClient<Entity>(Db); } }

        /// <summary>
        /// 获取所有
        /// </summary>
        /// <returns></returns>
        public virtual List<Entity> GetList()
        {
            try
            {
                return CurrentDb.GetList();
            }
            catch (MySqlException ex)
            {
                throw new BizException(ex.Message, 2);
            }
        }
        /// <summary>
        /// 根据表达式查询
        /// </summary>
        /// <returns></returns>
        public virtual List<Entity> GetListIn(Expression<Func<Entity, object>> inExpression = null, dynamic[] parmas = null)
        {
            try
            {
                if (inExpression == null || parmas == null)
                {
                    return null;
                }
                else
                {
                    return Db.Queryable<Entity>().In(inExpression, parmas).ToList();
                }
            }
            catch (MySqlException ex)
            {
                throw new BizException(ex.Message, 2);
            }
        }/// <summary>
         /// 根据表达式查询
         /// </summary>
         /// <returns></returns>
        public virtual List<Entity> GetList(Expression<Func<Entity, bool>> whereExpression)
        {
            try
            {
                return Db.Queryable<Entity>().Where(whereExpression).ToList();
            }
            catch (MySqlException ex)
            {
                throw new BizException(ex.Message, 2);
            }
        }
        /// <summary>
        /// 根据表达式查询
        /// </summary>
        /// <returns></returns>
        public virtual List<Entity> GetListWhereIn(Expression<Func<Entity, bool>> whereExpression = null, Expression<Func<Entity, object>> inExpression = null, dynamic[] parmas = null)
        {
            try
            {
                if (whereExpression == null && (inExpression == null || parmas == null))
                {
                    return null;
                }

                else if (whereExpression == null && inExpression != null && parmas != null)
                {
                    return Db.Queryable<Entity>().In(inExpression, parmas).ToList();
                }
                else
                {
                    if (whereExpression != null && (inExpression == null || parmas == null))
                    {
                        return Db.Queryable<Entity>().Where(whereExpression).ToList();
                    }
                    else
                    {
                        return Db.Queryable<Entity>().Where(whereExpression).In(inExpression, parmas).ToList();
                    }
                }
            }
            catch (MySqlException ex)
            {
                throw new BizException(ex.Message, 2);
            }

        }


        /// <summary>
        /// 根据表达式查询分页
        /// </summary>
        /// <returns></returns>
        public virtual List<Entity> GetPageList(Expression<Func<Entity, bool>> whereExpression, Pagination pagination)
        {
            try
            {
                int total = 0;
                List<Entity> list = Db.Queryable<Entity>().Where(whereExpression).ToPageList(pagination.currPage, pagination.pageSize, ref total);
                pagination.total = total;
                pagination.totalPage = (int)Math.Ceiling((double)pagination.total / (double)pagination.pageSize);
                return list;
            }
            catch (MySqlException ex)
            {
                throw new BizException(ex.Message, 2);
            }
        }


        /// <summary>
        /// 根据表达式查询分页
        /// </summary>
        /// <returns></returns>
        //public virtual List<Entity> GetPageList(Expression<Func<Entity, bool>> whereExpression, PageModel pageModel)
        //{
        //    return CurrentDb.GetPageList(whereExpression, pageModel);
        //}

        /// <summary>
        /// 根据表达式查询分页并排序
        /// </summary>
        /// <param name="whereExpression">it</param>
        /// <param name="pageModel"></param>
        /// <param name="orderByExpression">it=>it.id或者it=>new{it.id,it.name}</param>
        /// <param name="orderByType">OrderByType.Desc</param>
        /// <returns></returns>
        public virtual List<Entity> GetPageList(Expression<Func<Entity, bool>> whereExpression, Pagination pagination, Expression<Func<Entity, object>> orderByExpression = null, OrderByType orderByType = OrderByType.Asc)
        {
            try
            {
                int total = 0;
                List<Entity> list = Db.Queryable<Entity>().Where(whereExpression).OrderBy(orderByExpression, orderByType).ToPageList(pagination.currPage, pagination.pageSize, ref total);
                pagination.total = total;
                pagination.totalPage = (int)Math.Ceiling((double)pagination.total / (double)pagination.pageSize);
                return list;
            }
            catch (MySqlException ex)
            {
                throw new BizException(ex.Message, 2);
            }

        }


        /// <summary>
        /// 根据主键查询
        /// </summary>
        /// <returns></returns>
        public virtual Entity GetById(dynamic id)
        {
            try
            {
                return CurrentDb.GetById(id);
            }
            catch (MySqlException ex)
            {
                throw new BizException(ex.Message, 2);
            }
        }
        /// <summary>
        /// 根据主键查询
        /// </summary>
        /// <returns></returns>
        public virtual List<Entity> GetByIds(dynamic[] ids)
        {
            try
            {
                return Db.Queryable<Entity>().In("id", ids).ToList();
            }
            catch (MySqlException ex)
            {
                throw new BizException(ex.Message, 2);
            }
        }

        /// <summary>
        /// 根据主键删除
        /// </summary>
        /// <param name="id"></param>
        /// <returns></returns>
        public virtual bool Delete(dynamic id)
        {
            if (string.IsNullOrEmpty(id))
            {
                Console.WriteLine(string.Format("要删除的主键id不能为空值！"));
            }
            try
            {
                return CurrentDb.DeleteById(id);
            }
            catch (MySqlException ex)
            {
                throw new BizException(ex.Message, 2);
            }
        }


        /// <summary>
        /// 根据实体删除
        /// </summary>
        /// <param name="id"></param>
        /// <returns></returns>
        public virtual bool Delete(Entity data)
        {
            if (data == null)
            {
                Console.WriteLine(string.Format("要删除的实体对象不能为空值！"));
            }
            try
            {
                return CurrentDb.Delete(data);
            }
            catch (MySqlException ex)
            {
                throw new BizException(ex.Message, 2);
            }
        }

        /// <summary>
        /// 根据主键删除
        /// </summary>
        /// <param name="id"></param>
        /// <returns></returns>
        public virtual bool Delete(dynamic[] ids)
        {
            if (ids.Count() <= 0)
            {
                Console.WriteLine(string.Format("要删除的主键ids不能为空值！"));
            }
            try
            {
                return CurrentDb.AsDeleteable().In(ids).ExecuteCommand() > 0;
            }
            catch (MySqlException ex)
            {
                throw new BizException(ex.Message, 2);
            }
        }

        /// <summary>
        /// 根据表达式删除
        /// </summary>
        /// <param name="id"></param>
        /// <returns></returns>
        public virtual bool Delete(Expression<Func<Entity, bool>> whereExpression)
        {
            try
            {
                return CurrentDb.Delete(whereExpression);
            }
            catch (MySqlException ex)
            {
                throw new BizException(ex.Message, 2);
            }
        }


        /// <summary>
        /// 根据实体更新，实体需要有主键
        /// </summary>
        /// <param name="id"></param>
        /// <returns></returns>
        public virtual bool Update(Entity obj)
        {
            if (obj == null)
            {
                Console.WriteLine(string.Format("要更新的实体不能为空，必须带上主键！"));
            }
            try
            {
                return CurrentDb.Update(obj);
            }
            catch (MySqlException ex)
            {
                if (ex.Message.StartsWith("Duplicate entry"))
                {
                    string s = ex.Message.Replace("Duplicate entry", "").Split("for key")[0];
                    throw new BizException(s + "重复！", 2);
                }
                else
                {
                    throw new BizException(ex.Message, 2);
                }
            }
        }

        /// <summary>
        ///批量更新
        /// </summary>
        /// <param name="id"></param>
        /// <returns></returns>
        public virtual bool Update(List<Entity> objs)
        {
            if (objs.Count <= 0)
            {
                Console.WriteLine(string.Format("要批量更新的实体不能为空，必须带上主键！"));
            }
            try
            {
                return CurrentDb.UpdateRange(objs);
            }
            catch (MySqlException ex)
            {
                if (ex.Message.StartsWith("Duplicate entry"))
                {
                    string s = ex.Message.Replace("Duplicate entry", "").Split("for key")[0];
                    throw new BizException(s + "重复！", 2);
                }
                else
                {
                    throw new BizException(ex.Message, 2);
                }
            }
        }

        /// <summary>
        /// 插入
        /// </summary>
        /// <param name="id"></param>
        /// <returns></returns>
        public virtual bool Insert(Entity obj)
        {
            try
            {

                return CurrentDb.Insert(obj);
            }
            catch (MySqlException ex)
            {
                if (ex.Message.StartsWith("Duplicate entry"))
                {
                    string s = ex.Message.Replace("Duplicate entry", "").Split("for key")[0];
                    throw new BizException(s + "重复！", 2);
                }
                else
                {
                    throw new BizException(ex.Message, 2);
                }

            }
        }


        /// <summary>
        /// 批量
        /// </summary>
        /// <param name="id"></param>
        /// <returns></returns>
        public virtual bool Insert(List<Entity> objs)
        {
            try
            {
                return CurrentDb.InsertRange(objs);
            }
            catch (MySqlException ex)
            {
                if (ex.Message.StartsWith("Duplicate entry"))
                {
                    string s = ex.Message.Replace("Duplicate entry", "").Split("for key")[0];
                    throw new BizException(s + "重复！", 2);
                }
                else
                {
                    throw new BizException(ex.Message, 2);
                }
            }
        }
    }
}
