﻿using CMS.Context;
using CMS.Models.Source;
using CMS.Models.Target;
using Pomelo.AspNetCore.TimedJob;
using System;
using System.Collections.Generic;
using System.Linq;
using System.Linq.Expressions;
using System.Threading.Tasks;

namespace CMS.Services
{
    public class DataETLService: IBusinessService
    {
        private readonly SourceService sourceService;
        private readonly TargetService targetService;
        private readonly SourceCourseService sourceCourse;
        private readonly TargetCourseService targetCourse;
        private readonly SourceSchoolInfoService sourceSchoolInfo;
        private readonly TargetSchoolInfoService targetSchoolInfo;
        public DataETLService(SourceService _sourceService,TargetService _targetService, SourceCourseService _sourceCourse,
            TargetCourseService _targetCourse, SourceSchoolInfoService _sourceSchoolInfo,TargetSchoolInfoService _targetSchoolInfo) {
            sourceService = _sourceService;
            targetService = _targetService;
            sourceCourse = _sourceCourse;
            targetCourse = _targetCourse;
            sourceSchoolInfo = _sourceSchoolInfo;
            targetSchoolInfo = _targetSchoolInfo;
        }
        /// <summary>
        /// 抽取数据
        /// </summary>
        /// <returns></returns>
        public List<SourceExercise> ExtractData() {
            //Expression<Func<SourceExercise, bool>> linq = null;
            Expression<Func<SourceCourse, object>> linqCourse = null;
            Expression<Func<SourceExercise, object>> linqIn = null;
            Expression<Func<SourceSchoolInfo, object>> linqSchoolInfo = null;
            string[] schoolCode = {"3664000017","2164000815","2164000203","3164000162","3164001604","2164000070" };
            //获取多个学校基本信息
            linqSchoolInfo = linqSchoolInfo => linqSchoolInfo.SchoolCode;
            List<SourceSchoolInfo> info = sourceSchoolInfo.GetListWhereIn(null, linqSchoolInfo, schoolCode);
            //取对应ID集合
            string[] SID = info.Select(x => x.SchoolID).ToArray();
            //linq = linq => 1 == 1;
            linqIn = linqIn => linqIn.CourseNO;
            linqCourse = linqCourse => linqCourse.SchoolID;
            List<SourceCourse> courses = sourceCourse.GetListWhereIn(null,linqCourse,SID);
            string[] CourseNo = courses.Select(x => x.CourseNO).ToArray();
            return sourceService.GetListWhereIn(null, linqIn, CourseNo) ;
        }
        /// <summary>
        /// 转换数据
        /// </summary>
        /// <returns></returns>
        public List<TargetExercise> TransformData(List<SourceExercise> sourceExercises)
        {
            Expression<Func<SourceSchoolInfo, object>> linqSchoolInfo = null;
            Expression<Func<SourceCourse, object>> linqCourse = null;
            List<TargetExercise> targetExercises = new List<TargetExercise>();
            string[] schoolCode = { "3664000017", "2164000815", "2164000203", "3164000162", "3164001604", "2164000070" };
            //获取多个学校基本信息
            linqSchoolInfo = linqSchoolInfo => linqSchoolInfo.SchoolCode;
            List<SourceSchoolInfo> info = sourceSchoolInfo.GetListWhereIn(null, linqSchoolInfo, schoolCode);
            //取对应ID集合
            string[] SID = info.Select(x => x.SchoolID).ToArray();
            linqCourse = linqCourse => linqCourse.SchoolID;
            List<SourceCourse> courses = sourceCourse.GetListWhereIn(null, linqCourse, SID);

            sourceExercises.ForEach(x => {
                List<string> sid = courses.Where(c => c.CourseNO.Contains(x.CourseNO.ToString())).Select(c => c.SchoolID).ToList();
                List<string> sName = info.Where(s => s.SchoolID.Contains(sid[0])).Select(s => s.SchoolName).ToList();
                targetExercises.Add(new TargetExercise { ExNO = x.ExNO ,SchoolName = sName[0],MemberID=x.MemberID,CourseNO=x.CourseNO, ExType = x.ExType, ExName = x.ExName ,ExLink = x.ExLink,
                QNumber =x.QNumber,StuCount = x.StuCount,AnsNum = x.AnsNum,TrueNum = x.TrueNum,TrueRate = x.TrueRate,TotalSpendTime = x.TotalSpendTime,
                AvgSpendTime = x.AvgSpendTime,Rule = x.Rule,Status = x.Status,SERIALNUMBER = x.SERIALNUMBER,Description = x.Description,ReportSubject = x.ReportSubject});
            });
            return targetExercises;
        }
        /// <summary>
        /// 装载数据
        /// </summary>
        /// <returns></returns>
        public bool LoadData(List<TargetExercise> targetExercises)
        {
            //return true;
            return targetService.Insert(targetExercises);
        }
        /// <summary>
        /// Begin 起始时间；Interval执行时间间隔，单位是毫秒，建议使用以下格式，此处为6小时；
        /// SkipWhileExecuting是否等待上一个执行完成，true为等待；
        /// </summary>
        /// <param name="services"></param>
        [Invoke(Begin = "2018-09-21 00:00", Interval = 1000 * 60 * 60 * 1, SkipWhileExecuting = true)]
        public bool ETL() {
            List<SourceExercise> sourceExercises=  ExtractData();

            List<TargetExercise> targetExercises = TransformData(sourceExercises);
            return LoadData(targetExercises);
        }
    }
}
